// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#pragma once

#include <stddef.h>
#include <stdint.h>

#include <adbc.h>
#include <sqlite3.h>

#include "statement_reader.h"

struct SqliteDatabase {
  sqlite3* db;
  char* uri;
  size_t connection_count;
};

struct SqliteConnection {
  sqlite3* conn;
  char active_transaction;
  char load_extension;

  // Temporarily store an extension to load (need both entrypoint and path)
  char* extension_path;
};

struct SqliteStatement {
  sqlite3* conn;

  // -- Query state -----------------------------------------

  sqlite3_stmt* stmt;
  char prepared;
  char* query;
  size_t query_len;

  // -- Bind state ------------------------------------------
  struct AdbcSqliteBinder binder;

  // -- Ingest state ----------------------------------------
  char* target_catalog;
  char* target_table;
  char append;
  char temporary;

  // -- Query options ---------------------------------------
  int batch_size;
};
